;****************************************************************************
;
;    PROGRAM: CallBack.asm
;
;    PURPOSE: assembler support for countdos windows program
;
;    FUNCTIONS:
;	ConfirmCallBack()   entry point for the real-mode callback. Sets
;			    up a consistent state, and calls a C-routine
;			    in countdos.c
;
;	InstallConfirmProc() allocates a real-mode callback through DPMI, and
;			    stores the address in the TSR's data block
;
;	RemoveConfirmProc() de-allocates the DPMI real-mode callback, and
;			    zeroes out the pointer in the TSR data block
;
;    COMMENTS:
;
;
;****************************************************************************/
; COPYRIGHT:
;
;   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
;
;   You have a royalty-free right to use, modify, reproduce and
;   distribute the Sample Files (and/or any modified version) in
;   any way you find useful, provided that you agree that
;   Microsoft has no warranty obligations or liability for any
;   Sample Application Files which are modified.
;

.286

.MODEL SMALL
include cmacros.inc
include int31.inc

.DATA
ExternW <_Selector>		    ; variable declared in countdos.c

RM_Registers Real_Mode_Call_Struc <>

.CODE
ExternNP <ConfirmDelete>	    ; function declared in countdos.c

;****************************************************************************
;
;    FUNCTION: ConfirmCallBack()
;
;    PURPOSE:  Call-back routine to confirm delete operation
;
;    ENTRY:
;	Interrupts disabled
;	DS:SI	Selector:Offset of real-mode SS:SP
;	ES:DI	Selector:Offset of real mode call structure
;	SS:SP	Locked protected mode API stack (large enough for windows calls)
;
;    COMMENTS:
;	Set ds to the App's data segment, form a pointer to the file name
;	(which is stored on the real-mode stack) and call the actual
;	confirmation function.
;	Must then update the real-mode CS:IP, and set the flags according
;	to the confirmation function's return value: carry set if delete
;	NOT confirmed.
;
;****************************************************************************/
PUBLIC ConfirmCallBack
ConfirmCallBack PROC FAR
    push    di
    push    si
    push    ds
    push    es

    mov     ax, ds
    mov     dx, es
    mov     ds, dx
    add     si, 4			; first 4 bytes on stack = return addr.
    FarPtr  lpText,ax,si		; ax:si -> file name (on stack)

    sti 				; necessary before calling windows APIs
    ccall   ConfirmDelete,<lpText>

    pop     es
    pop     ds
    pop     si
    pop     di
    mov     dx, ax			; save return value in dx

    cld
    lodsw				; get real mode ret IP
    mov     es:[di.RealMode_IP], ax
    lodsw				; get real mode ret CS
    mov     es:[di.RealMode_CS], ax
    test    dx, dx			; Q: did ConfirmDelete return TRUE?
    jnz     short CCB_OK_Delete 	;   Y: must clear carry
    or	    WORD PTR es:[di.RealMode_Flags], 1	  ; N: set carry flag
    jmp     short CCB_Exit

CCB_OK_Delete:
    and     WORD PTR es:[di.RealMode_Flags], 0FFFEh ; clear carry flag

CCB_Exit:
    add     es:[di.RealMode_SP], 4
    iret

ConfirmCallBack ENDP


;****************************************************************************
;
;    FUNCTION: InstallConfirmProc()
;
;    PURPOSE:  Installs the Confirm Delete procedure
;
;    COMMENTS:
;	 To install the confirm delete YES/NO box, we must allocate a
;	 real-mode call-back, via DPMI services
;
;****************************************************************************/

cProc _InstallConfirmProc, <NEAR, PUBLIC>
cBegin

    push    ds
    push    es
    pusha

    mov     ah, Int31_Trans_Serv
    mov     al, Trans_Call_Back
    mov     dx, ds
    mov     es, dx
    mov     di, OFFSET RM_Registers	    ; es:di -> real mode structure
    mov     si, OFFSET ConfirmCallBack
    push    ds				    ; save ds...
    push    cs
    pop     ds				    ; ds:si -> procedure to call
    int     31h

    jc	    short ICP_Exit
    pop     ds				    ; restore ds
    mov     ax, [_Selector]
    mov     ds, ax
    mov     ds:[256*4], dx		    ; write address in TSR data block
    mov     ds:[256*4 + 2], cx

ICP_Exit:
    popa
    pop     es
    pop     ds

cEnd



;****************************************************************************
;
;    FUNCTION: RemoveConfirmProc()
;
;    PURPOSE:  Un-installs the Confirm Delete procedure
;
;    COMMENTS:
;	Must de-allocate the DPMI callback, and zero out the address in TSR
;
;****************************************************************************/

cProc _RemoveConfirmProc, <NEAR, PUBLIC>
cBegin
    pusha
    push    ds

    mov     ax, [_Selector]
    mov     ds, ax
    mov     cx, ds:[256*4 + 2]		; get rm callback address from TSR
    mov     dx, ds:[256*4]
    push    cx
    or	    cx, dx			; Q: is a real-mode callback installed?
    pop     cx
    jz	    short RCP_Done		;   N: can't un-install!
    mov     ah, Int31_Trans_Serv
    mov     al, Trans_Free_CB
    int     31h 			; de-allocate real-mode callback

    xor     ax, ax
    mov     ds:[256*4], ax		; write NULL ptr in TSR data block
    mov     ds:[256*4 + 2], ax

RCP_Done:
    pop     ds
    popa

cEnd

    END
