//**********************************************************************
//  File name: cddecli.h
//
//  Description: 
//            Declares the CDDEClient and CDDEClientConv
//            classes
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

#include <afxcoll.h>
#include <ddeml.h>
#include "map_do.h"

//**********************************************************************
//
//  class CDDEClient 
//
//  Purpose:
//  
//  CDDEClient is a class that manages the DDEML client.          
//
//  Derived from:  CObject
// 
//  Data Members:  
//     DWORD m_idInst;                // instance ID of DDEML client
//     CMapDWordToOb m_mapConv;       // map of HCONV to CDDEClientConv
//     static CDDEClient *m_pClient;  // static pointer to itself
//     PFNCALLBACK m_lpCallback;      // DDEML callback function
//
//  Member Functions:
//
// Constructors
//   CDDEClient();
//
// Destructor
//   virtual ~CDDEClient();
//
// Interface
//    BOOL DdeEnableCallback(UINT uCmd = 0);
//    UINT DdeGetLastError();
//    HSZ DdeCreateStringHandle(LPCSTR lpszString,int codepage = CP_WINANSI);
//    HDDEDATA DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
//                    DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd);
//    BOOL DdeFreeDataHandle(HDDEDATA hData);
//    BOOL DdeFreeStringHandle(HSZ hsz);
//    BOOL DdeKeepStringHandle(HSZ hsz);
//    DWORD DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax,
//                    int codepage = CP_WINANSI);
//    HCONV DdeReconnect(HCONV hConv);
//    BOOL DdeUnaccessData(HDDEDATA hData);
// 
// Callback handlers
//    virtual void OnRegister( HSZ hszBaseServName, HSZ hszInstServName );
//    virtual void OnUnregister( HSZ hszBaseServName, HSZ hszInstServName );
//
// DDEML Callback function
//    static HDDEDATA _export CALLBACK DDEClientCallback( UINT type, UINT fmt,
//             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
//             DWORD dwData2);
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CDDEClientConv;

class CDDEClient : public CObject
{
    DECLARE_DYNAMIC( CDDEClient )
    
    public:
    
 // Constructor
 
    CDDEClient();
    
 // Destructor
 
    virtual ~CDDEClient();
    
 // Initalization and Uninitialization
    
    virtual void Initialize() {DdeInitialize(0,0);}
    virtual void Uninitialize() {DdeUninitialize();}
    UINT DdeInitialize( DWORD afCmd=0, DWORD uRes=0);
    BOOL DdeUninitialize();
    
  
 // Interface
 
   
    BOOL DdeEnableCallback(UINT uCmd = 0);
    UINT DdeGetLastError();
    HSZ DdeCreateStringHandle(LPCSTR lpszString,int codepage = CP_WINANSI);
    HDDEDATA DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
                     DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd);
    BOOL DdeFreeDataHandle(HDDEDATA hData);
    BOOL DdeFreeStringHandle(HSZ hsz);
    BOOL DdeKeepStringHandle(HSZ hsz);

   
    DWORD DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax, int codepage = CP_WINANSI);
    HCONV DdeReconnect(HCONV hConv);

    BOOL DdeUnaccessData(HDDEDATA hData);
    
                 
 // Call back functions
 
    virtual void OnRegister( HSZ hszBaseServName, HSZ hszInstServName );
    virtual void OnUnregister( HSZ hszBaseServName, HSZ hszInstServName );
    
 // Implementation
 
    static CDDEClientConv * FromHandle(HCONV hConv);
    

    DWORD m_idInst;   
    CMapDWordToOb m_mapConv;
    static CDDEClient *m_pClient;
    private:
  
    // Callback function
    static HDDEDATA _export CALLBACK DDEClientCallback( UINT type, UINT fmt,
             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
             DWORD dwData2);
     
    PFNCALLBACK m_lpCallback;

};


//**********************************************************************
//
//  class CDDEClientConv 
//
//  Purpose:
//  
//  CDDEClientConv is a class that manages the DDEML client conversation.          
//
//  Derived from:  CObject
// 
//  Data Members:  
//    CDDEClient *m_pClient;    // Pointer to applications CDDEClient
//    HCONV m_hConv;            // Handle of the conversation
//    HSZ m_hszTopic;           // Topic of the conversation
//    HSZ m_hszService;         // Server we are talking with
//
//  Member Functions:
//
// Constructors
//
//   CDDEClientConv();
//
// Destructor
//
//   virtual ~CDDEClientConv();
//
// Interface
//
//    BOOL DdeConnect( LPCSTR hszService, LPCSTR hszTopic, 
//           CONVCONTEXT FAR *lpcc = NULL);
//    BOOL DdeEnableCallback(UINT uCmd = 0);
//    BOOL DdeAbandonTransaction(DWORD dwTransaction = NULL);
//    HDDEDATA DdeClientTransaction(void FAR *lpvData, DWORD cbData, 
//           HSZ hszItem, UINT uFmt, UINT uType, DWORD uTimeout, 
//           DWORD FAR *lpuResult = NULL);
//    BOOL DdeDisconnect();
//    BOOL DdePostAdvise(HSZ hszItem);
//    UINT DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo);
//    BOOL DdeSetUserHandle(DWORD id, DWORD hUser);
// 
// Callback handlers
//    virtual HDDEDATA OnAdvData( HSZ hszTopic, HSZ hszItem, 
//                      HDDEDATA hDataAdvise);
//    virtual void OnDisconnect( BOOL fSameInst);
//    virtual void OnXactComplete( HSZ hszTopic, HSZ hszItem, 
//                      HDDEDATA hDataXact, DWORD dwXactID, DWORD fwStatus);
//
//
//  Comments: Typically, you will need to derive a class from this one
//            which will define what to do with the data that the 
//            conversation receives.
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CDDEClientConv : public CObject
{
    DECLARE_DYNAMIC( CDDEClientConv )
    
    public:
    
    // Constructor
    
    CDDEClientConv();
    
    // Destructor
    
    virtual ~CDDEClientConv();
    
    // Interface
    
    BOOL DdeConnect( LPCSTR hszService, LPCSTR hszTopic, CONVCONTEXT FAR *lpcc = NULL);
    BOOL DdeEnableCallback(UINT uCmd = 0);
    BOOL DdeAbandonTransaction(DWORD dwTransaction = NULL);
    HDDEDATA DdeClientTransaction(void FAR *lpvData, DWORD cbData, HSZ hszItem,
                UINT uFmt, UINT uType, DWORD uTimeout, DWORD FAR *lpuResult = NULL);
    BOOL DdeDisconnect();
    BOOL DdePostAdvise(HSZ hszItem);
    UINT DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo);
    BOOL DdeSetUserHandle(DWORD id, DWORD hUser);
    
   // Call back functions
    
    virtual HDDEDATA OnAdvData( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataAdvise);
    virtual void OnDisconnect( BOOL fSameInst);
    virtual void OnXactComplete( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataXact,
                                DWORD dwXactID, DWORD fwStatus);
    // Implementation
    
    CDDEClient *m_pClient;
    HCONV m_hConv;
    HSZ m_hszTopic;
    HSZ m_hszService;
};
