//**********************************************************************
//  File name: cddeserv.h
//
//  Description: 
//            Declares the CDDEServer and CDDEServerConv
//            classes
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

#include <ddeml.h>
#include <afxcoll.h>
#include "map_do.h" 

//**********************************************************************
//
//  class CDDEServer 
//
//  Purpose:
//  
//  CDDEServer is a class that manages the DDEML server.          
//
//  Derived from:  CObject
// 
//  Data Members:  
//     DWORD m_idInst;                // instance ID of DDEML client
//     CMapDWordToOb m_mapConv;       // map of HCONV to CDDEClientConv
//     static CDDEClient *m_pServer;  // static pointer to itself
//     PFNCALLBACK m_lpCallback;      // DDEML callback function
//
//  Member Functions:
//
// Constructors
//    CDDEServer();
//
// Destructor
//    virtual ~CDDEServer();
//
// Initialization and Uninitialization
//    void Initialize() {DdeInitialize(0,0);}
//    void Uninitialize() {DdeUninitialize();}
//    UINT DdeInitialize( DWORD afCmd=0, DWORD uRes=0);
//    BOOL DdeUninitialize();
//
// Interface
//    BOOL DdePostAdvise(HSZ hszTopic, HSZ hszItem);
//    HDDEDATA DdeNameService( HSZ hszServ, HSZ hszRes, UINT afCmd);
//    BOOL DdeEnableCallback(UINT uCmd = 0);
//    UINT DdeGetLastError();
//    HSZ DdeCreateStringHandle(LPCSTR lpszString,int codepage = CP_WINANSI);
//    HDDEDATA DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
//                    DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd);
//    BOOL DdeFreeDataHandle(HDDEDATA hData);
//    BOOL DdeFreeStringHandle(HSZ hsz);
//    BOOL DdeKeepStringHandle(HSZ hsz);
//    DWORD DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax,
//                    int codepage = CP_WINANSI);
//    HCONV DdeReconnect(HCONV hConv);
//    BOOL DdeUnaccessData(HDDEDATA hData);
// 
// Callback handlers
//    virtual void OnRegister( HSZ hszBaseServName, HSZ hszInstServName );
//    virtual void OnUnregister( HSZ hszBaseServName, HSZ hszInstServName );
//    virtual BOOL OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
//    virtual HDDEDATA OnWildConnect(UINT wFmt,HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
//    virtual void OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,BOOL fSameInstance);    
//
// Implementation
//    static CDDEServerConv * FromHandle(HCONV hConv);
//
// DDEML Callback function
//    static HDDEDATA _export CALLBACK DDEClientCallback( UINT type, UINT fmt,
//             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
//             DWORD dwData2);
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CDDEServerConv;
class CDDEServer : public CObject
{
    DECLARE_DYNAMIC( CDDEServer )
    public:
    
 // Constructor
 
    CDDEServer();
    
 // Destructor
 
    virtual ~CDDEServer();
    
 // Initalization and Uninitialization
    
    void Initialize() {DdeInitialize(0,0);}
    void Uninitialize() {DdeUninitialize();}
    UINT DdeInitialize( DWORD afCmd=0, DWORD uRes=0);
    BOOL DdeUninitialize();
    
  
 // Interface
 
    BOOL DdePostAdvise(HSZ hszTopic, HSZ hszItem);
    HDDEDATA DdeNameService( HSZ hszServ, HSZ hszRes, UINT afCmd);
    BOOL DdeEnableCallback(UINT uCmd = 0);
    UINT DdeGetLastError();
    HSZ DdeCreateStringHandle(LPCSTR lpszString,int codepage = CP_WINANSI);
    HDDEDATA DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
                     DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd);
    BOOL DdeFreeDataHandle(HDDEDATA hData);
    BOOL DdeFreeStringHandle(HSZ hsz);
    BOOL DdeKeepStringHandle(HSZ hsz);

   
    DWORD DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax, int codepage = CP_WINANSI);
    HCONV DdeReconnect(HCONV hConv);

    BOOL DdeUnaccessData(HDDEDATA hData);
    
                 
 // Call back functions
 
    virtual void OnRegister( HSZ hszBaseServName, HSZ hszInstServName );
    virtual void OnUnregister( HSZ hszBaseServName, HSZ hszInstServName );
    virtual BOOL OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
    virtual HDDEDATA OnWildConnect(UINT wFmt,HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
    virtual void OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,BOOL fSameInstance);
    
 // Implementation
 
    static CDDEServerConv * FromHandle(HCONV hConv);
    

    DWORD m_idInst;   
    CMapDWordToOb m_mapConv;
    static CDDEServer *m_pServer;
    private:
  
    // Callback function
    static HDDEDATA _export CALLBACK DDEServerCallback( UINT type, UINT fmt,
             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
             DWORD dwData2);
     
    PFNCALLBACK m_lpCallback;

};

//**********************************************************************
//
//  class CDDEServerConv 
//
//  Purpose:
//  
//  CDDEServerConv is a class that manages the DDEML server conversation.          
//
//  Derived from:  CObject
// 
//  Data Members:  
//    CDDEServer *m_pServer;    // Pointer to applications CDDEServer
//    HCONV m_hConv;            // Handle of the conversation
//    HSZ m_hszTopic;           // Topic of the conversation
//    HSZ m_hszService;         // Server we are talking with
//
//  Member Functions:
//
// Constructors
//
//   CDDEServerConv();
//
// Destructor
//
//   virtual ~CDDEServerConv();
//
// Interface
//
//    void Create(HCONV hConv, HSZ hszTopic,HSZ hszService, BOOL fSameInstance);
//    BOOL DdeConnect( LPCSTR hszService, LPCSTR hszTopic, 
//           CONVCONTEXT FAR *lpcc = NULL);
//    BOOL DdeEnableCallback(UINT uCmd = 0);
//    BOOL DdeAbandonTransaction(DWORD dwTransaction = NULL);
//    HDDEDATA DdeClientTransaction(void FAR *lpvData, DWORD cbData, 
//           HSZ hszItem, UINT uFmt, UINT uType, DWORD uTimeout, 
//           DWORD FAR *lpuResult = NULL);
//    BOOL DdeDisconnect();
//    BOOL DdePostAdvise(HSZ hszItem);
//    UINT DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo);
//    BOOL DdeSetUserHandle(DWORD id, DWORD hUser);
//   
// Callback handlers
//
//    virtual UINT OnAdvData( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataAdvise);
//    virtual void OnDisconnect( BOOL fSameInst);
//    virtual HDDEDATA OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem);
//    virtual HDDEDATA OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq);
//    virtual BOOL OnAdvStart(HSZ hszTopic,HSZ hszItem);
//    virtual void OnAdvStop(HSZ hszTopic,HSZ hszItem);
//    virtual UINT OnPoke(HSZ hszTopic,HSZ hszItem, HDDEDATA hData);
//    virtual UINT OnExecute(HSZ hszTopic, HDDEDATA hData);
//
//  Comments: Typically, you will need to derive a class from this one
//            which will define what to do where to get the data for the
//            conversation.
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************
class CDDEServerConv : public CObject
{
    DECLARE_DYNAMIC( CDDEServerConv )
    
    public:
    CDDEServerConv();
    virtual ~CDDEServerConv();
    void Create(HCONV hConv, HSZ hszTopic,HSZ hszService, BOOL fSameInstance);
    BOOL DdeConnect( LPCSTR hszService, LPCSTR hszTopic, CONVCONTEXT FAR *lpcc = NULL);
    BOOL DdeEnableCallback(UINT uCmd = 0);
    BOOL DdeAbandonTransaction(DWORD dwTransaction = NULL);
    HDDEDATA DdeClientTransaction(void FAR *lpvData, DWORD cbData, HSZ hszItem,
                UINT uFmt, UINT uType, DWORD uTimeout, DWORD FAR *lpuResult = NULL);
    BOOL DdeDisconnect();
    BOOL DdePostAdvise(HSZ hszItem);
    UINT DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo);
    BOOL DdeSetUserHandle(DWORD id, DWORD hUser);
    
    // Call back functions
    
    virtual UINT OnAdvData( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataAdvise);
    virtual void OnDisconnect( BOOL fSameInst);
    virtual HDDEDATA OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem);
    virtual HDDEDATA OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq);
    virtual BOOL OnAdvStart(HSZ hszTopic,HSZ hszItem);
    virtual void OnAdvStop(HSZ hszTopic,HSZ hszItem);
    virtual UINT OnPoke(HSZ hszTopic,HSZ hszItem, HDDEDATA hData);
    virtual UINT OnExecute(HSZ hszTopic, HDDEDATA hData);
    
    // Implementation
    CDDEServer *m_pServer;
    HCONV m_hConv;
    HSZ m_hszTopic;
    HSZ m_hszService;
};
