//*************************************************************
//  File name: cdib.cpp
//
//  Description: 
//            Defines functions in the CDIB class.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//***********************************************************

// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//

#include <afxwin.h>
#include <afx.h>
#include "cdib.h"

#pragma code_seg("_CDIB")

IMPLEMENT_SERIAL(CDIB, CObject, 0)


//***********************************************************
//
//  CDIB :: CDIB
//
//  Purpose:
//              Copy constructor: creates a new CDIB object
//              based on the contents of an existing CDIB object.
//
//
//  Parameters:  const CDIB &
//     
//
//  Return: (void)
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
CDIB::CDIB(const CDIB& cdib)
{
   BYTE huge *lpCopy;
   BYTE huge *lp;

   long lDIBSize = ::GlobalSize(cdib.m_hDIB);

   m_hDIB = ::GlobalAlloc(GMEM_MOVEABLE,lDIBSize);

   ASSERT(m_hDIB);

   lpCopy = (BYTE huge *)::GlobalLock(m_hDIB);
   lp = (BYTE huge *)::GlobalLock(cdib.m_hDIB);

   while(lDIBSize--)
    *lpCopy++= *lp++;

   ::GlobalUnlock(m_hDIB);
   ::GlobalUnlock(cdib.m_hDIB);
}

//***********************************************************
//
//  CDIB :: operator=
//
//  Purpose:
//              Assignment operator: destroys contents of an 
//              existing CDIB object and then copies the contents
//              of another CDIB object into the newly freed object
//
//
//  Parameters:  const CDIB &
//     
//
//  Return: CDIB &
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
CDIB & CDIB::operator=(const CDIB& cdib)
{
   BYTE huge *lpCopy;
   BYTE huge *lp;

   DestroyDIB();
   long lDIBSize = ::GlobalSize(cdib.m_hDIB);

   m_hDIB = ::GlobalAlloc(GMEM_MOVEABLE,lDIBSize);
   ASSERT(m_hDIB);

   lpCopy = (BYTE huge *)::GlobalLock(m_hDIB);
   lp = (BYTE huge *)::GlobalLock(cdib.m_hDIB);

   while(lDIBSize--)
    *lpCopy++= *lp++;

   ::GlobalUnlock(m_hDIB);
   ::GlobalUnlock(cdib.m_hDIB);

   CPalette &pal = GetPalette();
   m_Pal.Attach(pal.Detach());

   return(*this);
}
   


//***********************************************************
//
//  CDIB :: CreateDIB
//
//  Purpose:
//              Creates a CDIB object based on a BITMAPINFO 
//              structure and a pointer to the bitmaps bits. 
//                                                         
//
//  Parameters:  const BITMAPINFO FAR *
//               const void HUGE *
//     
//
//  Return: (void)
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CDIB::CreateDIB(const BITMAPINFO FAR *lpbmi, const void huge *lpBits)
{
 long colorsize=0;
 long bitsize;
 BYTE huge *lpDIB;
 
 if(m_hDIB) ::DestroyDIB(m_hDIB);
 
 // Calculate DIB size
 if(((BITMAPINFOHEADER FAR *)lpbmi)->biBitCount != 24)
    colorsize = (1<((BITMAPINFOHEADER FAR *)lpbmi)->biBitCount);
 bitsize = ((BITMAPINFOHEADER FAR *)lpbmi)->biHeight * 
         WIDTHBYTES(((BITMAPINFOHEADER FAR *)lpbmi)->biBitCount *
         ((BITMAPINFOHEADER FAR *) lpbmi)->biWidth);
 
 // copy DIB 
 m_hDIB = ::GlobalAlloc(GMEM_MOVEABLE,sizeof(BITMAPINFOHEADER) + colorsize + bitsize);
 lpDIB = (BYTE huge *)::GlobalLock(m_hDIB);
 _fmemcpy(lpDIB,lpbmi, (size_t)(sizeof(BITMAPINFOHEADER) + colorsize));
 _fmemcpy(lpDIB + (sizeof(BITMAPINFOHEADER) + colorsize), lpBits,(size_t)bitsize);
 ::GlobalUnlock(m_hDIB);
 
}

//***********************************************************
//
//  CDIB :: PaintDIB
//
//  Purpose:
//              Paints a CDIB into a CDC.
//                                                         
//
//  Parameters: const CDC &   // device context to paint
//              const CRect   // part of the DIB to use
//              const CPoint  // where to put the image on the CDC
//              HPALETTE Pal  // palette to use when painting.
//     
//
//  Return: (void)
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CDIB::PaintDIB(const CDC& cdc, const CRect rSrc, const CPoint pDest, HPALETTE Pal)
{
    ASSERT(m_hDIB);

    CRect rDest(pDest.x,pDest.y,pDest.x+(int)GetWidth(),pDest.y+(int)GetHeight());     
    ::PaintDIB(cdc.GetSafeHdc(),rDest,m_hDIB,rSrc,Pal);
}

void huge * CDIB::FindDIBits()
{
   LPSTR lpDIB;
   void huge *lpBits;

   ASSERT(m_hDIB);

   lpDIB = ::GlobalLock(m_hDIB);
   lpBits = FindDIBBits(lpDIB);
   ::GlobalUnlock(m_hDIB);
   return lpBits;
}

//***********************************************************
//
//  CDIB :: Serialize
//
//  Purpose:
//              Writes a CDIB object to disk or reads it
//              in from disk.
//                                                         
//
//  Parameters: CArchive  // archive to store CDIB 
//     
//
//  Return: (void)
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CDIB::Serialize(CArchive& archive)
{

 if(archive.IsStoring())
   {
   long colorsize=0;
   long bitsize;
   long dwBytes;

   ASSERT(m_hDIB);

   BYTE huge *hpBuffer = (BYTE huge *)::GlobalLock(m_hDIB);   // make a huge pointer to the data
 
   // Calculate DIB size
   //if(((BITMAPINFOHEADER FAR *)(hpBuffer))->biBitCount != 24)
     //colorsize = (1<((BITMAPINFOHEADER FAR *)(hpBuffer))->biBitCount);


   bitsize = ((BITMAPINFOHEADER FAR *)(hpBuffer))->biHeight * 
         WIDTHBYTES(((BITMAPINFOHEADER FAR *)(hpBuffer))->biBitCount * 
                ((BITMAPINFOHEADER FAR *)(hpBuffer))->biWidth);
   dwBytes = bitsize + ::PaletteSize((LPSTR)hpBuffer)
             + sizeof(BITMAPINFOHEADER);
   
   // save the DIB size
   archive << dwBytes;

   DWORD dwBytesTmp = dwBytes;       // Save # of bytes for return value
   
   /*
    * Write out the data in 32768 byte chunks.
    */

   while (dwBytes > 32767)
   {
     // _lwrite(iFileHandle, (LPSTR)hpBuffer, (WORD)32768)
     archive.Write(hpBuffer,(UINT)32768);
      
      dwBytes -= 32768;
      hpBuffer += 32768;
   }

   /* Write out the last chunk (which is < 32768 bytes) */
   //_lwrite(iFileHandle, (LPSTR)hpBuffer, (WORD)dwBytes)
   archive.Write(hpBuffer,(UINT)dwBytes);
                                                           
   ::GlobalUnlock(m_hDIB);
   }
 else
   {
   // reading the DIB from disk
   long dwBytes;
   
   archive >> dwBytes;
   
   m_hDIB = ::GlobalAlloc(GMEM_MOVEABLE, dwBytes);
   
   BYTE huge *hpBits =(BYTE huge * )::GlobalLock(m_hDIB);
   
   while(dwBytes > 32767)
      {
      archive.Read(hpBits,32768);
      
      dwBytes -= 32768;
      hpBits += 32768;
      }
   
   // read remaining bytes
   archive.Read(hpBits, (UINT)dwBytes);
   ::GlobalUnlock(m_hDIB);

   CPalette &pal = GetPalette();
   m_Pal.Attach(pal.Detach());
   }                                               
}
