//*************************************************************
//  File name: cdib.h
//
//  Description: 
//            Declares the CDIB class.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//*************************************************************

#include <afxwin.h>
#include <afxcoll.h>

extern "C"
{
#include "dibapi.h"
}



//***********************************************************
//
//  class CDIB 
//
//  Purpose:
//  
//  CDIB is a class that manages creating, displaying and 
//  storing Device Independent Bitmaps (DIB).            
//
//  Derived from:  CObject
// 
//  Data Members:  
//         HDIB m_hDIB;         // Handle to the DIB
//         CPalette m_Palette;  // Palette for DIB
//
//  Member Functions:
//
// Constructors
//   CDIB();
//   CDIB(const CDIB &);
//
// Destructor
//   virtual ~CDIB();
//   
// Asignment operator
//   CDIB & operator=(const CDIB &);
//
// Input functions
//   void LoadDIB(const LPSTR lpszFile)
//   void CBitmapToDIB(const CBitmap& bitmap, const CPalette& pal);
//   void CreateDIB(const HDIB hDIB);
//   void CreateDIB(const BITMAPINFO FAR *,const void huge *);
//   
// Output functions
//   void SaveDIB(const LPSTR lpszFile)
//   CBitmap& DIBtoCBitmap(CPalette pal)
//   void PaintDIB(const CDC& cdc ,const CRect rSrc, const CRect rDest, HPALETTE hPal)
//   void PaintDIB(const CDC& ,const CRect, const CPoint, HPALETTE hPal);
//   
// Access functions
//   long GetWidth();
//   long GetHeight();
//   void huge *FindDIBits();
//   CPalette& GetPalette();
//
// Cleanup
//   void DestroyDIB();
// 
// Serialization
//   virtual void Serialize(CArchive& archive);
//
//  
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//*************************************************************

class CDIB : public CObject
{
   DECLARE_SERIAL(CDIB)
 
   public:
   
   HDIB         m_hDIB;
   CPalette     m_Pal;
   
   // Constructors
   CDIB() {m_hDIB = NULL;}
   CDIB(const CDIB&);

   CDIB & operator=(const CDIB &);

   // Destructor
   virtual ~CDIB() {if(m_hDIB) ::DestroyDIB(m_hDIB);if(m_Pal.m_hObject) m_Pal.DeleteObject();}
   
   // Input functions
   void LoadDIB(const LPSTR lpszFile) {if(m_hDIB) ::DestroyDIB(m_hDIB);
                                       m_hDIB = ::LoadDIB(lpszFile);}
   void CBitmapToDIB(const CBitmap& bitmap, const CPalette& pal)
             { if(m_hDIB) ::DestroyDIB(m_hDIB);
             m_hDIB = BitmapToDIB(bitmap.GetSafeHandle(),
                                    pal.GetSafeHandle());}
   void CreateDIB(const HDIB hDIB) { if(m_hDIB) ::DestroyDIB(m_hDIB);
                                    m_hDIB = hDIB;}
   void CreateDIB(const BITMAPINFO FAR *,const void huge *);
   
   // Output functions
   void SaveDIB(const LPSTR lpszFile) {::SaveDIB(m_hDIB, lpszFile);}
   CBitmap& DIBtoCBitmap(CPalette &pal) {return *(CBitmap::FromHandle(
                           DIBToBitmap(m_hDIB, pal.GetSafeHandle())));}
   void PaintDIB(const CDC& cdc ,const CRect rSrc, const CRect rDest,
                 HPALETTE Pal) {
         ::PaintDIB(cdc.GetSafeHdc(),rDest,m_hDIB,rSrc,Pal); }
   void PaintDIB(const CDC& ,const CRect, const CPoint, HPALETTE Pal);
   
   // Access functions
   long GetWidth() {ASSERT(m_hDIB); return DIBWidth(GlobalLock(m_hDIB)); GlobalUnlock(m_hDIB);}
   long GetHeight() {ASSERT(m_hDIB); return DIBHeight(GlobalLock(m_hDIB));GlobalUnlock(m_hDIB);}
   void huge *FindDIBits();
   CPalette& GetPalette() {ASSERT(m_hDIB); return *(CPalette::FromHandle(
                                     CreateDIBPalette(m_hDIB)));}
                                     
   void DestroyDIB() { if(m_hDIB) ::DestroyDIB(m_hDIB); m_hDIB = NULL;
                       m_Pal.DeleteObject();}
   
   
   // Serialization
   virtual void Serialize(CArchive& archive);
   
};
