//*************************************************************
//  File name: mfcdib.cpp
//
//  Description:  Excercises the CDIB class.  The application can load, save,
//                display, copy, paste, and serialize CDIB objects.
//
//  History:    Date       Author              Comment
//              05/15/92   Hung H. Nguyen      Created
//                         Brian Scott
//
// Written by Microsoft Product Support Services, Windows Developer Support
// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
//*************************************************************
#include <afxwin.h>
#include <afxdlgs.h>
#include "global.h"
#include "mfcdib.h"

#pragma code_seg("_MFCDIB")

CApp App;   // Constructs the main frame window



//***********************************************************
//
//  CMainWindow :: CMainWindow()
//
//  Purpose:  Creates the main frame window of the app and calculates the
//            maximum size of the client area.  In case the DIB is larger
//            than the client area, we want to size the window to be no
//            larger than the screen size.
//                                                         
//  Parameters:  None
//
//  Return: None
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
CMainWindow::CMainWindow()
{
    cxScreen = ::GetSystemMetrics(SM_CXSCREEN)-2*GetSystemMetrics(SM_CXFRAME);
    cyScreen = ::GetSystemMetrics(SM_CYSCREEN)-2*GetSystemMetrics(SM_CYFRAME)-
               ::GetSystemMetrics(SM_CYCAPTION)-GetSystemMetrics(SM_CYMENU);


    Create("AfxFrameWnd", "CDIB Sample App", WS_OVERLAPPEDWINDOW,
           rectDefault, NULL, "MainMenu");

}

//*************************************************************
//
//  Message map for CMainWindow
//
//*************************************************************
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
    ON_WM_PAINT()
    ON_WM_PALETTECHANGED()
    ON_WM_QUERYNEWPALETTE()
    ON_WM_INITMENU()
    ON_COMMAND(IDM_OPEN, OnOpen)
    ON_COMMAND(IDM_SAVE, OnSave)
    ON_COMMAND(IDM_EXIT, OnExit)
    ON_COMMAND(IDM_COPYDIB, OnCopyDIB)
    ON_COMMAND(IDM_PASTEDIB, OnPasteDIB)
    ON_COMMAND(IDM_COPYBM, OnCopyBM)
    ON_COMMAND(IDM_PASTEBM, OnPasteBM)
    ON_COMMAND(IDM_READ, OnRead)
    ON_COMMAND(IDM_WRITE, OnWrite)
    ON_COMMAND(IDM_ABOUT,        OnAbout)
    ON_COMMAND(IDM_INSTRUCTIONS, OnInstructions)
END_MESSAGE_MAP()


//***********************************************************
//
//  CMainWindow :: OnPaint()
//
//  Purpose:  Paints the DIB onto the client area.  This handles the WM_PAINT
//            message.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:  The painting is slow because a DIB is used.  To increase the
//             speed, use DDBs by converting a CDIB to a CBitmap and use
//             BitBlt() or StretchBlt() for painting.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnPaint()
{
    CPaintDC    dc(this);
    RECT        rect;
    CPoint      point(0,0);

    if(App.m_cDIB.m_hDIB)
    {
        rect.top = rect.left = 0;
        rect.bottom = (int)App.m_cDIB.GetHeight();
        rect.right = (int)App.m_cDIB.GetWidth();

        // Call CDIB::PaintDIB to display the DIB
        App.m_cDIB.PaintDIB(dc, rect, point, App.m_cDIB.m_Pal.m_hObject);
    }
}


//***********************************************************
//
//  CMainWindow :: OnOpen()
//
//  Purpose:  Creates a CFileDialog object for the Open common dialog and
//            lets the user select a .BMP file to load.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:  This routine does not have code to handle a bad .BMP file.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnOpen()
{
    CString     strFile('\0', MAX_FILENAME);
    CFileDialog FileOpen(TRUE, "bmp", NULL, OFN_HIDEREADONLY |
                         OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST,
                         "Bitmaps (*.bmp)|*.bmp||",
                         this);

    if(FileOpen.DoModal() == IDOK)
    {
        // Grab the filename with the extension
        strFile = FileOpen.GetFileName();
        strFile += ".";
        strFile += FileOpen.GetFileExt();

        App.m_cDIB.LoadDIB(strFile.GetBuffer(0));

        SizeWindow();   // Resize the client area to the size of the DIB

        strFile.ReleaseBuffer();

        CPalette        &Pal = App.m_cDIB.GetPalette();

        // Delete the old palette and attach the DIB's palette to the CDIB
        // object
        App.m_cDIB.m_Pal.DeleteObject();
        App.m_cDIB.m_Pal.Attach(Pal.Detach());

        Invalidate();   // Invalidate the client area for repainting
    }
}


//***********************************************************
//
//  CMainWindow :: SizeWindow()
//
//  Purpose:  Resizes the window so that client area fits the DIB exactly.
//            If the DIB is greater than the maximum size of the client
//            area (with respect to the max of the screen), the DIB will
//            be clipped.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::SizeWindow(void)
{
    RECT        rect;
    CPoint      Org;
    int         nWidth, nHeight;

    // Get the dimensions of the DIB
    nWidth = (int)App.m_cDIB.GetWidth();
    nHeight = (int)App.m_cDIB.GetHeight();

    // We want to retain the origin of the window if possible
    GetWindowRect(&rect);
    Org.x = rect.left;
    Org.y = rect.top;
    if(nWidth >= cxScreen)
    {
        nWidth = cxScreen;
        Org.x = 0;
    }
    if(nHeight >= cyScreen)
    {
        nHeight = cyScreen;
        Org.y = 0;
    }

    rect.left = rect.top = 0;
    rect.right = nWidth;
    rect.bottom = nHeight;

    // Adjusts the window to accomodate the new size of the client area
    ::AdjustWindowRect(&rect, WS_OVERLAPPEDWINDOW, TRUE);

    // Display the window at the old origin if possible
    MoveWindow(Org.x, Org.y, rect.right - rect.left,
               rect.bottom - rect.top);
}

//***********************************************************
//
//  CMainWindow :: OnSave()
//
//  Purpose:  Creates a CFileDialog for the SaveAs common dialog and lets
//            the user save the DIB.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnSave()
{
    if(App.m_cDIB.m_hDIB)
    {
        CString     strFile('\0', MAX_FILENAME);
        CFileDialog FileSave(FALSE, "bmp", NULL, OFN_HIDEREADONLY,
                             "Bitmaps (*.bmp)|*.bmp|All Files (*.*)|*.*||",
                             this);

        if(FileSave.DoModal() == IDOK)
        {
            strFile = FileSave.GetFileName();
            strFile += ".bmp";
            App.m_cDIB.SaveDIB(strFile.GetBuffer(0));
            strFile.ReleaseBuffer();
        }
    }
}


//***********************************************************
//
//  CMainWindow :: OnExit()
//
//  Purpose:  Destroys the main window when the user selects to exit.
//                                                         
//  Parameters:
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnExit()
{
    DestroyWindow();
}


//***********************************************************
//
//  CMainWindow :: OnCopyDIB()
//
//  Purpose:  If a DIB is valid, call CopyToClipboard() with no parameters
//            to copy the DIB to the clipboard using CF_DIB format.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnCopyDIB()
{
    if(!App.m_cDIB.m_hDIB)
        MessageBox("Nothing to copy!");
    else
        CopyToClipboard();
}


//***********************************************************
//
//  CMainWindow :: OnPasteDIB()
//
//  Purpose:  Calls PasteFromClipboard() with no parameter to specify pasting
//            from the clipboard using CF_DIB format.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnPasteDIB()
{
    PasteFromClipboard();
}


//***********************************************************
//
//  CMainWindow :: OnCopyBM()
//
//  Purpose:  Calls CopyToClipboard() to paste the DIB onto the clipboard
//            using CF_BITMAP format.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnCopyBM()
{
    if(!App.m_cDIB.m_hDIB)
        MessageBox("Nothing to copy!");
    else
        CopyToClipboard(FALSE);
}


//***********************************************************
//
//  CMainWindow :: OnPasteBM()
//
//  Purpose:  Calls PasteFromClipboard() to paste from the clipboard using
//            CF_BITMAP format.
//
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnPasteBM()
{
    PasteFromClipboard(FALSE);
}


//***********************************************************
//
//  CMainWindow :: CopyToClipboard()
//
//  Purpose:  Copies the DIB to the clipboard.  The default format is CF_DIB.
//            Overide this by passing in FALSE which will use CF_BITMAP.
//                                                         
//  Parameters:  BOOL bDIB
//
//  Return: BOOL
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
BOOL CMainWindow::CopyToClipboard(BOOL bDIB)
{

    if(!OpenClipboard())
    {
        MessageBox("Can't open clipboard!");
        return(NULL);
    }

    if(bDIB)    // use CF_DIB format
    {
        CDIB    DIB(App.m_cDIB);
        ::SetClipboardData(CF_DIB, DIB.m_hDIB);
        DIB.m_hDIB = NULL;
    }
    else        // use CF_BITMAP format
    {

        //Convert the CDIB to a CBitmap object
        CBitmap&    Bitmap = App.m_cDIB.DIBtoCBitmap(App.m_cDIB.m_Pal);

        ::SetClipboardData(CF_BITMAP, Bitmap.Detach());

        // Clipboard now owns it so avoid calling the dtor
        Bitmap.m_hObject = NULL;
    }

    ::CloseClipboard();
    return(TRUE);
}


//***********************************************************
//
//  CMainWindow :: PasteFromClipboard()
//
//  Purpose:  Pastes from the clipboard.  Default is to grab a DIB (CF_DIB).
//            Overide this by passing in FALSE to grab a Bitmap (CF_BITMAP).
//            If a Bitmap is pasted, it will be converted to a CDIB object.
//                                                         
//  Parameters:  BOOL bDIB
//
//  Return: BOOL
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
BOOL CMainWindow::PasteFromClipboard(BOOL bDIB)
{
    if(!OpenClipboard())
        return(NULL);

    if(bDIB)
    {
        if(::IsClipboardFormatAvailable(CF_DIB))
        {
            CDIB        tmpDIB;

            // Grab the DIB from the clipboard
            tmpDIB.m_hDIB = ::GetClipboardData(CF_DIB);

            // Use operator = to assign it to our m_cDIB data member
            App.m_cDIB = tmpDIB;
            tmpDIB.m_hDIB = NULL;  //clipboard owns memory, don't call dtor
        }
        else
        {
            MessageBox("Clipboard does not contain CF_DIB!");
            return(NULL);
        }
    }
    else
    {
        if(::IsClipboardFormatAvailable(CF_BITMAP))
        {

            CBitmap     Bitmap;
            CPalette    Palette;

            Bitmap.Attach(::GetClipboardData(CF_BITMAP));

            // Grab the palette if available or else use the default
            if(::IsClipboardFormatAvailable(CF_PALETTE))
                Palette.Attach(::GetClipboardData(CF_PALETTE));
            else
                Palette.Attach(::GetStockObject(DEFAULT_PALETTE));

            // Convert the CBitmap to CDIB object
            App.m_cDIB.CBitmapToDIB(Bitmap, Palette);

            Palette.Detach();   // Palette embeded into CDIB, don't call dtor
            Bitmap.Detach();    // Clipboard owns data, don't call dtor
        }
        else
        {
            MessageBox("Clipboard does not contain CF_BITMAP!");
            return(NULL);
        }
    }

    CPalette    &Pal = App.m_cDIB.GetPalette();

    // Delete the old palette and grab the new one
    App.m_cDIB.m_Pal.DeleteObject();
    App.m_cDIB.m_Pal.Attach(Pal.Detach());

    // Resize the window and invalidate the client area for repainting
    SizeWindow();
    Invalidate(TRUE);
    ::CloseClipboard();
    return(TRUE);
}


//***********************************************************
//
//  CMainWindow :: OnRead()
//
//  Purpose:  Reads serialized CDIB object from disk.
//                                                         
//  Parameters: None
//
//  Return: (void)
//
//  Comments:  This is a simple example of serializing one CDIB object.
//             Complex network of CDIB objects can also be used.  See
//             MFC's CArchive class.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnRead(void)
{
    CString     strFile('\0', MAX_FILENAME);
    CFileDialog FileRead(TRUE, "ser", NULL, OFN_HIDEREADONLY |
                         OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST,
                         "Serialized Files (*.ser)|*.ser||",
                         this);

    if(FileRead.DoModal() == IDOK)      // Use common dialogs
    {
        strFile = FileRead.GetPathName();

        // Open the file and archive for reading
        CFile       file(strFile.GetBuffer(0), CFile::modeRead);
        CArchive    arc(&file, CArchive::load);
        CDIB        *tmpDIB;

        // Destroy the current DIB
        App.m_cDIB.DestroyDIB();

        // Read the CDIB object into a temp
        arc >> (CObject*&)tmpDIB;

        // Use the operator = to assign the temp CDIB object to the app's
        App.m_cDIB = *tmpDIB;

        if(tmpDIB->m_hDIB)
            delete tmpDIB;

        strFile.ReleaseBuffer();

        // Close the file and archive
        arc.Close();
        file.Close();

        // Resize the window and invalidate the client area for repainting
        SizeWindow();
        Invalidate(TRUE);
    }
}


//***********************************************************
//
//  CMainWindow :: OnWrite()
//
//  Purpose:  Writes CDIB object to disk.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:  This is a simple example of serializing one CDIB object.
//             Complex network of CDIB objects can also be used.  See
//             MFC's CArchive class.
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnWrite(void)
{
    CString     strFile('\0', MAX_FILENAME);
    CFileDialog FileWrite(FALSE, "ser", NULL, OFN_HIDEREADONLY |
                         OFN_OVERWRITEPROMPT,
                         "Serialized Files (*.ser)|*.ser||",
                         this);

    if(FileWrite.DoModal() == IDOK)     // Use common dialogs
    {
        strFile = FileWrite.GetPathName();

        // Open the file and archive for writing
        CFile       file(strFile.GetBuffer(0), CFile::modeWrite |
                         CFile::modeCreate);
        CArchive    arc(&file, CArchive::store);

        // Write the object out to disk
        arc << (CObject*)&App.m_cDIB;

        strFile.ReleaseBuffer();

        // Close the archive and file
        arc.Close();
        file.Close();
    }
}


//***********************************************************
//
//  CMainWindow :: OnPaletteChanged()
//
//  Purpose:  If the system palette changes, we need to grab our palette
//            and remap the colors.  This handles the WM_PALETTECHANGED
//            message.
//                                                         
//  Parameters:  CWnd *pWnd
//
//  Return: (void)
//
//
//  Comments:
//
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnPaletteChanged(CWnd * pWnd)
{
    if(App.m_cDIB.m_hDIB)
    {
        CWindowDC   dc(App.m_pMainWnd);
        CPalette    *OldPal;

        if(pWnd != App.m_pMainWnd)
        {
            OldPal = dc.SelectPalette(&App.m_cDIB.m_Pal, FALSE);
            dc.RealizePalette();
            Invalidate(TRUE);
            dc.SelectPalette(OldPal, FALSE);
        }
    }
}


//***********************************************************
//
//  CMainWindow :: OnQueryNewPalette()
//
//  Purpose:  If we get the focus, we want to ensure that our palette is
//            the foreground palette
//                                                         
//  Parameters:  None
//
//  Return: BOOL
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
BOOL CMainWindow::OnQueryNewPalette()
{
    if(App.m_cDIB.m_hDIB)
    {
        CWindowDC   dc(App.m_pMainWnd);
        CPalette    *OldPal;

        OldPal = dc.SelectPalette(&App.m_cDIB.m_Pal, FALSE);
        dc.RealizePalette();
        dc.SelectPalette(OldPal, FALSE);
        Invalidate(TRUE);
    }
    return(TRUE);
}


//***********************************************************
//
//  CMainWindow :: OnInitMenu()
//
//  Purpose:  This will enable and disable menu items. This handles the
//            WM_INITMENU message.
//                                                         
//  Parameters:  CMenu *pMenu
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnInitMenu(CMenu *pMenu)
{
    if(!App.m_cDIB.m_hDIB)
    {
        pMenu->EnableMenuItem(IDM_SAVE, MF_GRAYED);
        pMenu->EnableMenuItem(IDM_COPYDIB, MF_GRAYED);
        pMenu->EnableMenuItem(IDM_COPYBM, MF_GRAYED);
        pMenu->EnableMenuItem(IDM_WRITE, MF_GRAYED);
    }
    else
    {
        pMenu->EnableMenuItem(IDM_SAVE, MF_ENABLED);
        pMenu->EnableMenuItem(IDM_COPYDIB, MF_ENABLED);
        pMenu->EnableMenuItem(IDM_COPYBM, MF_ENABLED);
        pMenu->EnableMenuItem(IDM_WRITE, MF_ENABLED);
    }
    if(::IsClipboardFormatAvailable(CF_DIB))
        pMenu->EnableMenuItem(IDM_PASTEDIB, MF_ENABLED);
    else
        pMenu->EnableMenuItem(IDM_PASTEDIB, MF_GRAYED);

    if(::IsClipboardFormatAvailable(CF_BITMAP))
        pMenu->EnableMenuItem(IDM_PASTEBM, MF_ENABLED);
    else
        pMenu->EnableMenuItem(IDM_PASTEBM, MF_GRAYED);
}


//***********************************************************
//
//  CMainWindow :: OnAbout
//
//  Purpose:  Displays a modal dialog box for credits.
//                                                         
//  Parameters:  None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnAbout()
{
	CModalDialog about( "AboutBox", this );
	about.DoModal();
}


//***********************************************************
//
//  CMainWindow :: OnInstruction()
//
//  Purpose:  Displays a modal dialog box for instructions
//
//  Parameters: None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
void CMainWindow::OnInstructions()
{
	CInstructionBox InstructBox( "Instructions", this );
	InstructBox.DoModal();
}

//***********************************************************
//
//  CInstructionBox :: OnInitDialog
//
//  Purpose:  Setup the instruction dialog box.  Handles the WM_INITDIALOG
//            message.
//
//  Parameters: None
//
//  Return: (void)
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
BOOL CInstructionBox::OnInitDialog()
{
    CClientDC dc( this );
    int nHeight;

    m_Text  = "This is a sample application of a program written using MFC. ";
    m_Text += "The sample demonstrates one technique of implementing a CDIB ";
    m_Text += "class.  The application will display, save, copy, paste, and ";
    m_Text += "serialize device independent bitmaps (DIBs).";
    
    SetDlgItemText( ID_INSTBOX,m_Text );
    SetDlgItemText( ID_TITLE, "Purpose" );

    nHeight = ( dc.GetDeviceCaps( LOGPIXELSY ) * 16 )/72;
    m_TitleFont.CreateFont( nHeight, 0, 0, 0, FW_BOLD, 0, 0, 0, ANSI_CHARSET,
                            OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
                            FF_DONTCARE, (LPSTR)"Roman" );

    SendDlgItemMessage( ID_TITLE, WM_SETFONT, m_TitleFont.GetSafeHandle(), TRUE );

    return ( TRUE );
}

//***********************************************************
//
//  CApp :: InitInstance()
//
//  Purpose:  This function is called when the object is created. Data
//            members can be initialized here. The main window is created.
//            If all is successful, we return TRUE
//
//  Parameters: None
//
//  Return: BOOL
//
//  Comments:
//
//  History:    Date       Author              Comment
//              05/15/92   Brian Scott         Created
//                         Hung H. Nguyen
//
//*************************************************************
BOOL CApp::InitInstance()
{
    TRACE( "Sample CDIB Application\n" );

    m_pMainWnd = new CMainWindow();
    m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return ( TRUE );
}
