//***************************************************************************
//
//  APPLICATION:
//      modeless.exe
//
//
//  AUTHOR:
//      Developer Support, Microsoft Product Support Services.
//
//
//  PURPOSE:
//      This MFC sample creates and displays a modeless dailog box.
//
//
//	DESIGN:
//      For a modeless dialog box object, a class is derived from CDialog.
//      The main window object contains a pointer to this modeless dialog
//      object.  When the user selects the menu item to display the modeless
//      dialog, the object is created and displayed, and the menu item is
//      disabled.  When the dialog is closed, the it is destroyed using
//      DestroyWindow() and its memory is deleted.  A message is also posted 
//      to the main window so it could enable its menu item that is used to
//      display the modeless dialog.
//
//      Three derived classes are used in this sample app:
//          CModelessDialog : public CDialog
//          CTheApp         : public CWinApp
//          CMainWindow     : public CFrameWnd
//
//	
//  HISTORY:
//      Date        Author      Comment
//      --------    --------    ---------
//      07/15/92    IAG         Created
//
// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
//
//***************************************************************************
#include "modeless.h"
#include "resource.h"

//******** Global Decalrations

CTheApp TheApp;


///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CMainWindow
//
//  BASE CLASS: 
//      CFrameWnd
//
//  MEMBER FUNCTIONS:
//      CMainWindow()
//      OnModeless()
//      OnModelessMessage()
//
//  DESCRIPTION:
//      Main window of the application.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// CMainWindow():   Creates the main window.
//
// Parameters:      None.

CMainWindow::CMainWindow()
{
	Create( NULL, "MFC Modeless Dialog Box Application",
		    WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );

}


///////////////////////////////////////////////////////////////////////////
// OnModeless():    Displays the modeless dialog box.
//
// Parameters:      None.
//
// Returns:         Void.

void CMainWindow::OnModeless()
{
    // Disable the menu item so a second instance of the modelss dialog
    // cannot be invoked.
    GetMenu()->EnableMenuItem( IDM_MODELESS, MF_GRAYED );
	m_pModeless = new CModelessDialog( this );
}


///////////////////////////////////////////////////////////////////////////
// OnModelessMessage(): This function is called when a WM_MODELESS_MESSAGE
//                      message is sent to the CMainWindow object.  This
//                      message is sent so the main window can enable the
//                      menu item that invokes the modeless dialog.
//
// Parameters:          in: WPARAM wParam - 16-bit signed value passed as a 
//                                          parameter to a window procedure 
//                                          or callback function.
//
//                      in: LPARAM lParam - 32-bit signed value passed as a 
//                                          parameter to a window procedure 
//                                          or callback function.
//
// Returns:             Void.

LONG CMainWindow::OnModelessMessage( WPARAM wParam, LPARAM lParam )
{
    return (LONG) GetMenu()->EnableMenuItem( IDM_MODELESS, MF_ENABLED );
}


BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	ON_MESSAGE( WM_MODELESS_MESSAGE, OnModelessMessage )
	ON_COMMAND( IDM_MODELESS, OnModeless )
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CModelessDialog
//
//  BASE CLASS: 
//      CDialog
//
//  MEMBER FUNCTIONS:
//      CModelessDialog()
//      PostNcDestory()
//      OnClose()
//
//  DESCRIPTION:
//      Modeless dialog box object.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// CModelessDialog():   Creates the modeless dialog box.
//
// Parameters:          in: CWnd* pParent - Pointer to the parent of the dialog.

CModelessDialog::CModelessDialog( CWnd* pParent )
{
    Create( "ModelessBox", pParent );
}


///////////////////////////////////////////////////////////////////////////
// OnClose():           This function is called when the user closes
//                      the dialog box, either using the CLOSE button
//                      or ESC.  It posts a message to its parent so
//                      the parent can enable the menu item.
//
// Parameters:          None.
//
// Returns:             Void.

void CModelessDialog::OnClose()
{
    // This message is sent to the parent so it can enable the menu item.
    // The dialog can also do it directly but keeping the OOP methodology in 
    // mind, we will let the parent do this task.
    GetParent()->PostMessage( WM_MODELESS_MESSAGE );

    DestroyWindow();
}


///////////////////////////////////////////////////////////////////////////
// PostNcDestroy():     This function is called after the window has been
//                      destroyed.  Therefore, it is used for deleting
//                      the object's memory.
//
// Parameters:          None.
//
// Returns:             Void.

void CModelessDialog::PostNcDestroy()
{
	delete this;
}


BEGIN_MESSAGE_MAP( CModelessDialog, CDialog )
	ON_COMMAND( IDD_CLOSE, OnClose )
	ON_COMMAND( IDCANCEL, OnClose )
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CTheApp
//
//  BASE CLASS: 
//      CWinApp
//
//  MEMBER FUNCTIONS:
//      InitInstance()   
//
//  DESCRIPTION:
//      This is the application object.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// InitInstance():  - Instance initialization.
//                  - Creates main window object.
//
// Parameters:      None.
//
// Returns:         TRUE if initialization is successful; otherwise FALSE.

BOOL CTheApp::InitInstance()
{
	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return TRUE;
}


