//***************************************************************************
//
//  APPLICATION:
//      subclass.exe
//
//
//  AUTHOR:
//      Developer Support, Microsoft Product Support Services.
//
//
//  PURPOSE:
//      This sample subclasses an edit control of a dialog box and
//      makes it accept either numbers or letters.
//	
//
//	DESIGN:
//      Six derived classes are used:
//          CSubclassedEdit : public CEdit
//          CSubNumberEdit  : public CSubclassedEdit
//          CSubLetterEdit  : public CSubclassedEdit
//          CSubEditDialog  : public CModalDialog
//          CTheApp         : public CWinApp
//          CMainWindow     : public CFrameWnd
//
//      To subclass a window, the window procedure of that window is replaced
//      by AfxWndProc, and the window is attached to a CWnd object.  MFC
//      provides two functions to subclass a window: SubclassWindow() and the
//      SubclassDlgitem().  These functions store the old window procedure in
//      the location returned by GetSuperWndProc().
//      
//      To change the behavior of the target window, the CWnd object's
//      desired functions are overloaded.
//      
//      A subclassed windows behavior can be changed on the fly by subclassing
//      it with a different CWnd object.  But before calling the subclassing
//      function on that window, it should first be "UnSubclassed".
//      MFC does not provide a function to unsubclass a window.  Therefore,
//      we have to this task on our own by calling ::SetWindowLong() to replace
//      the AfxWndProc by the original window procedure; and then detaching
//      the CWnd object from the target window.
//      
//      In SUBCLASS.EXE, a dialog box's edit control is subclassed to have two
//      different behaviors--either accepting numbers or accepting letters.
//      A class, CSubcalssedEdit, is derived from CEdit, and UnSubclass() is
//      declared as its member function.  Two other classes are derived from
//      CSubclassedEdit to provide the two different behaviors for the edit
//      control.  The classes are CSubNumberEdit and CSubLetterEdit.  The
//      OnChar function is overloaded for both of these classes.
//
//      The dialog box class is derived from CModalDialog and it has two member
//      data objects of CSubNumberEdit and CSubLetterEdit.
//      
//
//	
//  HISTORY:
//      Date        Author      Comment
//      --------    --------    ---------
//      06/20/92    IG          Created
//
//  Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//
//***************************************************************************

#include "subclass.h"
#include "resource.h"

//******** Global Decalrations

CTheApp TheApp;



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CSubclassedEdit
//
//  BASE CLASS: 
//      CEdit
//
//  MEMBER FUNCTIONS:
//      UnSubclass()    
//
//  DESCRIPTION:
//      This class declares a new function, called UnSubclass(), that is 
//      used to unsubclass an edit control object derived from this class.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** UnSubclass():    It is used to unsubclass the edit control
//**** Parameters:      None.
//**** Returns:         TRUE if successful; otherwise FALSE.

BOOL CSubclassedEdit::UnSubclass()
{
    WNDPROC     OldWndProc;
    WNDPROC*    lplpfn = GetSuperWndProcAddr();

    if ( !*lplpfn )
        return FALSE;

    // Set the original window procedure
    OldWndProc = (WNDPROC)::SetWindowLong( m_hWnd, GWL_WNDPROC, (LONG) *lplpfn );

    // A subclassed window's procedure is always AfxWndProc.
    // If this is not TRUE, then it's not a subclassed window.
    if ( OldWndProc != AfxWndProc )
        return FALSE;

    Detach();

    return TRUE;
}


///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CSubNumberEdit
//
//  BASE CLASS: 
//      CSubclassedEdit
//
//  MEMBER FUNCTIONS:
//      OnChar()
//
//  DESCRIPTION:
//      This class only accepts numbers.  This is achieved by overloading
//      the OnChar() function.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** OnChar():        Only numbers are allowed to be entered in the edit control.
//**** Parameters:      Refer to manual for this function.
//**** Returns:         Void.

void CSubNumberEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if ((nChar >= '0' && nChar <= '9') || nChar == VK_BACK ) 
		CEdit::OnChar(nChar, nRepCnt, nFlags);  // permitted
    else
		MessageBeep( MB_ICONEXCLAMATION );
}


BEGIN_MESSAGE_MAP( CSubNumberEdit, CSubclassedEdit)
	ON_WM_CHAR()
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CSubLetterEdit
//
//  BASE CLASS: 
//      CSubclassedEdit
//
//  MEMBER FUNCTIONS:
//      OnChar()
//
//  DESCRIPTION:
//      This class only accepts letters.  This is achieved by overloading
//      the OnChar() function.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** OnChar():        Only letters are allowed to be entered in the edit control.
//**** Parameters:      Refer to manual for this function.
//**** Returns:         Void.

void CSubLetterEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if ((nChar >= 'A' && nChar <= 'Z') || (nChar == VK_BACK) 
    || (nChar  >= 'a' && nChar <= 'z') ) 
		CEdit::OnChar(nChar, nRepCnt, nFlags);  // permitted
    else
		MessageBeep( MB_ICONEXCLAMATION );
}


BEGIN_MESSAGE_MAP( CSubLetterEdit, CSubclassedEdit)
	ON_WM_CHAR()
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CSubEditDialog
//
//  BASE CLASS: 
//      CModalDialog
//
//  MEMBER FUNCTIONS:
//      OnInitDialog()
//      OnLetter()
//      OnNumber()
//      OnClose()
//
//  DESCRIPTION:
//      This is the dialog that contains the sunbclassed edit control.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** OnInitDialog():  Dialog initialization.
//**** Parameters:      Refer to manual for this function.
//**** Returns:         TRUE, indicating successful dialog initialization;
//                      otherwise FALSE.

BOOL CSubEditDialog::OnInitDialog ()
{
    // Subclass the edit control to accept numbers.
	return ( m_NumberEdit.SubclassDlgItem( IDD_EDIT, this ));
}


///////////////////////////////////////////////////////////////////////////
//**** OnNumber():      Changes the edit control input mode to accept numbers.
//**** Parameters:      None.
//**** Returns:         Void.

void CSubEditDialog::OnNumber ()
{
    // Unsubclass the edit control.
    if ( !m_LetterEdit.UnSubclass() )
        return;

    // Subclass the edit control to accept numbers.
	m_NumberEdit.SubclassDlgItem( IDD_EDIT, this );

    // Set the dialog controls to reflect the change. 
    GetDlgItem( IDD_TEXT )->SetWindowText( "This edit control accepts only numbers." );
    GetDlgItem( IDD_MODE )->SetWindowText( "N&umbers:" );
    GetDlgItem( IDD_NUMBER )->EnableWindow( FALSE );
    GetDlgItem( IDD_LETTER )->EnableWindow( TRUE );
    m_NumberEdit.SetWindowText( "\0" );
    m_NumberEdit.SetFocus();
}


///////////////////////////////////////////////////////////////////////////
//**** OnLetter():      Changes the edit control input mode to accept letters.
//**** Parameters:      None.
//**** Returns:         Void.

void CSubEditDialog::OnLetter ()
{
    // Unsubclass the edit control.
    if ( !m_NumberEdit.UnSubclass() )
        return;

    // Subclass the edit control to accept letters.
	m_LetterEdit.SubclassDlgItem( IDD_EDIT, this );

    // Set the dialog controls to reflect the change. 
    GetDlgItem( IDD_TEXT )->SetWindowText( "This edit control accepts only letters." );
    GetDlgItem( IDD_MODE )->SetWindowText( "L&etters:" );
    GetDlgItem( IDD_NUMBER )->EnableWindow( TRUE );
    GetDlgItem( IDD_LETTER )->EnableWindow( FALSE );
    m_LetterEdit.SetWindowText( "\0" );
    m_LetterEdit.SetFocus();
}


///////////////////////////////////////////////////////////////////////////
//**** OnClose():       - Ends the dialog box.
//                      - Displays the content of the edit control.
//**** Parameters:      None.
//**** Returns:         Void.

void CSubEditDialog::OnClose ()
{
	CString String;

    // Get the edit control's text to display it in the message box.
    if ( m_NumberEdit.m_hWnd )
    	m_NumberEdit.GetWindowText( String );
    else
    	m_LetterEdit.GetWindowText( String );

	EndDialog( IDOK );

    GetParent()->MessageBox( String, "Edit Control's Text", MB_OK );
}


BEGIN_MESSAGE_MAP( CSubEditDialog, CModalDialog )
	ON_COMMAND( IDD_NUMBER, OnNumber )
	ON_COMMAND( IDD_LETTER, OnLetter )
	ON_COMMAND( IDD_CLOSE, OnClose )
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CMainWindow
//
//  BASE CLASS: 
//      CFrameWnd
//
//  MEMBER FUNCTIONS:
//      CMainWindow()
//      OnAbout()
//      OnEditDialog()
//
//  DESCRIPTION:
//      Main window of the application.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** CMainWindow():   Creates the main window.
//**** Parameters:      None.

CMainWindow::CMainWindow()
{
	Create( NULL, "MFC Subclassing Application",
		WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );
}


///////////////////////////////////////////////////////////////////////////
//**** OnAbout():       Displays the ABOUT dialog box.
//**** Parameters:      None.
//**** Returns:         Void.

void CMainWindow::OnAbout()
{
	CModalDialog about( "AboutBox" );
	about.DoModal();
}


///////////////////////////////////////////////////////////////////////////
//**** OnEditDialog():  Displays the dialog that contains the subcalssed
//                      edit control.
//**** Parameters:      None.
//**** Returns:         Void.

void CMainWindow::OnEditDialog()
{
	CSubEditDialog SubEditDialog;
	SubEditDialog.DoModal();
}


BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	ON_COMMAND( IDM_ABOUT, OnAbout )
	ON_COMMAND( IDM_EDITDLG, OnEditDialog )
END_MESSAGE_MAP()



///////////////////////////////////////////////////////////////////////////
//
//  CLASS:
//      CTheApp
//
//  BASE CLASS: 
//      CWinApp
//
//  MEMBER FUNCTIONS:
//      InitInstance()   
//
//  DESCRIPTION:
//      This is the application object.
//
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//**** InitInstance():  - Instance initialization.
//                      - Creates main window object.
//**** Parameters:      None.
//**** Returns:         TRUE if initialization is successful; otherwise FALSE.

BOOL CTheApp::InitInstance()
{
	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

