;****************************************************************************
;
;    PROGRAM: TSR.asm
;
;    PURPOSE: well-behaved, Windows-aware TSR which communicates with a
;		Windows application
;
;    COMMENTS:
;
;	There are two parts to this file: the 'permanent' code and data,
;	which remains in the system after the TSR DOS call, and the
;	initialization code and data which is discarded after use.
;
;	The TSR will only install once in the system. If another copy is
;	detected, one of four actions will be taken, based on the command
;	line:
;	    /r parameter: internal stats are reset
;	    /d parameter: deletion confirmation is toggled on/off
;	    /u parameter: TSR is uninstalled and removed from system
;	    no params (or unknown params): internal stats are printed
;
;	The resident portion hooks int21 and counts the number of DOS calls.
;	It also hooks int2F for access to various Windows APIs.
;
;	When Windows starts up, the internal data block is instanced, so that
;	each VM (or each task if the task switcher rather than WIN386 is
;	present) has its own copy of the stats, as well as its own pointer
;	to a deletion-confirmation function.
;
;	The TSR will then respond to another int2F to request that Windows
;	execute a given Windows Application. The pathname of the application
;	is obtained from the environment block: it assumes that the TSR is
;	the DOS stub of a windows app.
;
;	The application is invoked with a command line giving the seg:offset
;	address of the internal stats block.
;
;	When deletion confirmation is enabled (when the pointer is non-NULL),
;	the int21h trapping code calls the specified function before chaining
;	to DOS, for calls 13h and 41h (ie: file delete). If the function
;	returns with carry set, the call is never passed to DOS.
;
;	Which function is used to confirm delete operations depends on the
;	context: in real-mode (or V86 mode), the TSR uses its own internal
;	routine, which uses simple DOS I/O calls.
;	In the Windows system VM, however, the TSR depends on its associated
;	Windows app to bring up a message box. This requires that the Windows
;	app. use DPMI to allocate a real-mode call-back. It then writes the
;	address to this call-back at the appropriate location in the TSR.
;
;****************************************************************************

; COPYRIGHT:
;
;   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
;
;   You have a royalty-free right to use, modify, reproduce and
;   distribute the Sample Files (and/or any modified version) in
;   any way you find useful, provided that you agree that
;   Microsoft has no warranty obligations or liability for any
;   Sample Application Files which are modified.
;

.286

.MODEL small

include int2fapi.inc

.STACK
.CODE

;
; all code and data in the code segment (simulate a TINY model)
;
ASSUME	cs:_TEXT
ASSUME	ds:_TEXT

_dummy_var  dw	0		    ; make sure nothing has address 0

;
; Next handler addresses for both trapped interrupts
;
PUBLIC Int_2Fh_Chain_Addr
Int_2Fh_Chain_Addr  LABEL DWORD
Int_2Fh_Chain_Off   dw	?
Int_2Fh_Chain_Seg   dw	?

PUBLIC Int_21h_Chain_Addr
Int_21h_Chain_Addr  LABEL DWORD
Int_21h_Chain_Off   dw	?
Int_21h_Chain_Seg   dw	?

;
; The following two structures are used to designate the block of data
; that must be instanced.
;
PUBLIC Startup_Info
Startup_Info Win386_Startup_Info_Struc <>

Instance_Info	Instance_Item_Struc <>
_end_of_list	dd  0

;
; This structure will be passed to Windows to indicate that the TSR is
; Windows-aware. It also contains pointers to the pathname of the application
; that Windows must execute, and to the internal data block.
;
PUBLIC TSR_Info
TSR_Info    TSR_Info_Struc  <>

PUBLIC Exec_Path_Name
Exec_Path_Name	db  128 dup (0)

;
; TSR identification string. Used by Windows if an error message is required,
; also used by the TSR itself to determine if it is already installed in the
; system.
; Must be preceded by a word-count.
;
PUBLIC My_ID_Block
My_ID_Block	dw  ?
My_Name 	db  'Sample TSR / Windows App',0
My_Name_End	LABEL BYTE

;
; Internal data block. Windows imposes no structure.
; We have:
;	256 DWORDs to count each DOS call
;	a call-back address field; if non-zero, points to a routine which
;	    will verify file delete operations
;	a flag to inhibit file-deletion checking during Windows load time
;
;	These 3 fields must be in a single block, so that the Windows App
;	can access them through a single selector. In fact, the App depends
;	on the exact structure and placement of these fields - eg: it accesses
;	the flag as offset 257*4 from the start of the data block.
;	*** Any change in the data block entails changing the Windows App.***
;
PUBLIC My_Data_Block
My_Data_Block	dd  256 dup (0)
SIZE_My_Data	EQU ($-My_Data_Block)
Confirm_Delete_CB dd  0
SIZE_Data_Block    EQU ($-My_Data_Block)
Windows_Starting dw 0

;**************************************************************************
;
; Interrupt handler for int2Fh
;
; Respond to the various APIs.
;
;**************************************************************************

Int_2Fh_Handler PROC NEAR

    cmp     ah, W386_Int_Multiplex
    jne     short not_W386_Multiplex

    cmp     al, W386_TSR_Identify
    je	    short Query_TSR_Info
    cmp     al, W386_Startup
    je	    short Win386_Startup

not_W386_Multiplex:
    ; not a WIN386 API. Check if it is the task-switcher (used by WIN286)
    cmp     ax, 4B05h
    je	    short TaskSwitch_Instance
    jmp     cs:[Int_2Fh_Chain_Addr]

Query_TSR_Info:
    ; return a pointer to the TSR structure
    mov     WORD PTR cs:[TSR_Info.TSR_Next], di
    mov     WORD PTR cs:[TSR_Info.TSR_Next+2], es
    mov     bx, cs
    mov     es, bx
    mov     di, OFFSET TSR_Info
    jmp     cs:[Int_2Fh_Chain_Addr]

Win386_Startup:
TaskSwitch_Instance:
    ; return a pointer to the instance data structure. Note that for this
    ; API, we must first chain to the other interrupt handlers, and link
    ; our structure to the list when the chained handlers return.
    pushf				; required because handlers will iret
    call    cs:[Int_2Fh_Chain_Addr]	; chain to other handlers

    mov     WORD PTR cs:[Startup_Info.SIS_Next_Ptr], bx
    mov     WORD PTR cs:[Startup_Info.SIS_Next_Ptr+2], es
    mov     bx, cs
    mov     es, bx
    mov     bx, OFFSET Startup_Info
    ;
    ; Set the flag to inhibit deletion confirmation during Windows load
    ; (because Windows may have to delete some files - a confirmation request
    ; would be most inapropriate at this point!)
    ; It is the Windows App's responsibility to reset this flag when it starts.
    ;
    mov     WORD PTR cs:[Windows_Starting], -1 ; set the flag
    iret

Int_2Fh_Handler ENDP


;**************************************************************************
;
; Interrupt handler for int21h (DOS calls)
;
; increment the appropriate DWORD in the internal data block, to keep
; a count of the DOS calls.
;
; For file delete operations, if deletion confirmation is enabled (and
; if Windows isn't in its load phase), call the appropriate function.
; In this case, only pass the int 21h to DOS if the function returns with
; carry clear.
;**************************************************************************

Int_21h_Handler PROC NEAR

    push    di
    push    ax
    push    cx
    ; DOS call # is in AH. We want to multiply this number by 4 to obtain
    ; an index into the DWORD table. So we shift ax Right by 8, to transfer
    ; AH into AL, the AX Left by 2 to multiply by 4.
    ; Net result: shift AX right by 6.
    xor     al, al
    mov     cl, 6
    shr     ax, cl
    mov     di, ax
    add     cs:WORD PTR [My_Data_Block+di], 1	; increment low-word
    adc     cs:WORD PTR [My_Data_Block+2+di], 0 ; inc. high-word if necessary
    pop     cx
    pop     ax
    pop     di
    cmp     WORD PTR cs:[Confirm_Delete_CB], 0	; Q: is ptr to function NULL?
    jne     short I21_Check_Delete		;   N: confirm. enabled
    cmp     WORD PTR cs:[Confirm_Delete_CB+2], 0 ;Q: 2nd part of DWORD ptr NULL?
    je	    short I21_Exit			;   Y: NULL ptr, no confirm.

I21_Check_Delete:
    cmp     WORD PTR cs:[Windows_Starting], 0	; Q: are we just starting up?
    jne     short I21_Exit			;   Y: no deletion checking now
    cmp     ah, 41h
    je	    I21_Func_41
    cmp     ah, 13h
    je	    I21_Func_13

I21_Exit:
    jmp     cs:[Int_21h_Chain_Addr]		; pass call to next handler

I21_Func_41:
;
; Prepare to call the deletion confirmation function. Copy the file name
; (pointed to by ds:dx) onto the stack. Only copy the file name, not the
; entire path, ie: copy from the last separator to the terminating NULL.
;
    pusha
    push    es
    push    ds
    sub     sp, 14				; leave room for 8.3 + NULL

    push    ds
    pop     es
    mov     di, dx				; es:di -> ASCIIZ filename
    xor     al, al				; prepare to search for 0
    mov     cx, -1				; search very far!
    cld
    repne scasb
    dec     di					; point to terminating 0
    std 					; prepare to search backwards
    mov     al, '\'                             ; search for separator
    mov     cx, 0Ch				; max of 12 (8+'.'+3) chars
    repne scasb
    jnz     short I21_F41_Copy
    add     di, 2				; di -> 1st char after separator

I21_F41_Copy:
    mov     si, di				; ds:si -> filename
    push    ss
    pop     es
    mov     di, sp				; es:di -> space on stack
    cld 					; go forward again
I21_F41_Copy_Loop:
    lodsb
    stosb
    test    al, al
    jnz     I21_F41_Copy_Loop
    jmp     I21_Done_Copy

I21_Func_13:
;
; Prepare to call deletion confirmation routine: copy the filename
; (contained at offset 1 of the FCB; ds:dx -> FCB). This is not an ASCIIZ
; string, it is 8.3 blank-padded.
;
    pusha
    push    es
    push    ds
    sub     sp, 14				; leave room for 8.3 + NULL

    mov     si, dx
    inc     si					; ds:si -> filename in FCB

    push    ss
    pop     es
    mov     di, sp				; es:di -> space on stack
    cld
    mov     cx, 8				; name is 8 chars. max

I21_F13_Copy_Name:
    lodsb
    cmp     al, ' '				; Q: blank padding?
    je	    short I21_F13_Copy_End_Loop 	;   Y: don't copy blanks
    stosb
I21_F13_Copy_End_Loop:
    loop    I21_F13_Copy_Name

I21_F13_Separator:
    cmp     BYTE PTR ds:[si], ' '		; Q: 1st char. of ext. blank?
    je	    short I21_F13_Done_Copy		;   Y: don't print '.'

    mov     al, '.'
    stosb
    mov     cx, 3				; extension is 3 chars. long

I21_F13_Copy_Ext:
    lodsb
    cmp     al, ' '				; Q: blank padding?
    je	    short I21_F13_Done_Copy		;   Y: done copying filename
    stosb
    loop    I21_F13_Copy_Ext

I21_F13_Done_Copy:
    xor     al, al
    stosb

;
; both code paths (function 13h and function 41h) merge here:
; stack contains filename
;
PUBLIC I21_Done_Copy
I21_Done_Copy:
    cld 					; safer to reset this!
    call    cs:[Confirm_Delete_CB]
    jc	    short I21_Dont_Delete
    add     sp, 14
    pop     ds
    pop     es
    popa
; carry is clear: chain to next handler (will eventually get to DOS and delete)
    jmp     cs:[Int_21h_Chain_Addr]

I21_Dont_Delete:
; carry set: don't delete (ie: don't chain to next handler), return error
; error codes: for function 13, al = FF, for function 41, ax = 5 (access denied)
; must simulate an iret without popping the flags
    add     sp, 14
    pop     ds
    pop     es
    popa
    mov     al, 0FFh
    cmp     ah, 41h
    jne     short I21_Simul_Iret
    mov     ax, 05h

I21_Simul_Iret:
    sti
    retf 2

Int_21h_Handler ENDP


;**************************************************************************
;
; Confirm Delete routine
;
; For each delete file call, verify with the user that this is really
; desired.
;
; entry:
;	file name (ASCIIZ) is on stack
;	all registers (except sp!) can be trashed
;
; exit:
;	if carry clear, delete is confirmed
;	if carry set: do not delete
;
;**************************************************************************

Confirm_Delete_msg  db	'Do you want to delete file: $'
Confirm_Delete_msg2 db	' ? [Y or N] $'

Confirm_Delete PROC FAR
    mov     ax, cs
    mov     ds, ax
    mov     ah, 09h
    mov     dx, OFFSET Confirm_Delete_msg
    int     21h

    mov     ah, 02h			    ; prepare to output char. by char.
    mov     bx, sp
    add     bx, 4			    ; ss:bx -> filename

CD_loop:
    mov     dl, ss:[bx] 		    ; get next char
    test    dl, dl			    ; Q: terminating NULL?
    jz	    short CD_done_print 	    ;	Y: done printing filename
    int     21h 			    ; char. I/O DOS call
    inc     bx
    jmp     CD_loop

CD_done_print:
    mov     ah, 09h			    ; print char. string function
    mov     dx, OFFSET Confirm_Delete_msg2
    int     21h

    mov     ah, 01h			    ; char. input function
    int     21h
    cmp     al, 'y'
    je	    short CD_OK_Delete
    cmp     al, 'Y'
    je	    short CD_OK_Delete

; not 'y' or 'Y' : don't delete, ie: set the carry
    stc
    jmp     short CD_Exit

CD_OK_Delete:
    clc

CD_Exit:
    mov     ah, 02h			    ; print CR/LF to clean up display
    mov     dl, 13
    int     21h
    mov     dl, 10
    int     21h

    retf

Confirm_Delete ENDP


Init_Code_Starts    LABEL   BYTE


;**************************************************************************
;
; hex_to_ascii : converts a word into its 4-byte ascii representation
;
; entry:
;	dx = word to convert
;	es:di->string which will contain ascii representation
;
;**************************************************************************

_hex_to_ascii_table   db  '0123456789ABCDEF'

PUBLIC hex_to_ascii
hex_to_ascii	PROC NEAR
    push    ax
    push    cx
    push    si
    add     di, 3
    std 			; start at end of string and work backwards
    mov     cx, 4
_convert_loop:
    mov     si, dx
    and     si, 0Fh		; only keep 1 nibble
    mov     al, BYTE PTR [_hex_to_ascii_table+si]
    stosb
    shr     dx, 1		; next nibble...
    shr     dx, 1		; ...on the 8086, can only shift by
    shr     dx, 1		; ...1 or cl, but cx is already in
    shr     dx, 1		; ...use, so we're stuck with 4 instructions
    loop    _convert_loop
    cld
    inc     di
    pop     si
    pop     cx
    pop     ax
    ret
hex_to_ascii	ENDP


;**************************************************************************
;
; Initialization : main entry point for TSR
;
; First set up some vital variables (eg: PSP address and size). Then check if
; TSR is already installed in system (issue an int2F call - the same one
; that Windows issues).
;
; If TSR isn't in system yet, install it.
; Otherwise parse arguments and take appropriate action.
;
;**************************************************************************

PUBLIC Initialization
Initialization PROC NEAR

; Store address and size of the PSP
    mov     ax, es
    mov     bx, cs
    mov     ds, bx			    ; set ds to code & data segment
    mov     WORD PTR [TSR_info.TSR_PSP_Segment], ax
    sub     bx, ax			    ; size (in paras) of PSP
    mov     WORD PTR [PSP_Size], bx

; Initialize length of ID string
    mov     WORD PTR [My_ID_Block], OFFSET My_Name_End - OFFSET My_Name

;
; broadcast an int2Fh, get list of responding TSRs
;
    xor     di, di
    mov     es, di
    mov     ah, W386_Int_Multiplex
    mov     al, W386_TSR_Identify
    int     2Fh

    mov     ax, es	    ; Check es:di -> first TSR_Info_Struc
    or	    ax, di	    ; Q: did any TSR respond?
    jnz     check_installed ;	Y: check if this TSR is already installed
    jmp     Install_TSR     ;	N: go to next phase of TSR initalization

check_installed:
    cld
;
; Walk list of TSRs, check ID to see if we are already installed
;
PUBLIC walk_list
walk_list:
    ; es:di -> responding TSR's TSR_Info_Struc
    lds     si, DWORD PTR es:[di.TSR_ID_Block]	; ds:si -> its ID block
    mov     ax, ds
    or	    ax, si				; Q: is pointer NULL?
    jz	    short next_TSR			;   Y: no ptr, go to next TSR
						;   N: compare ID strings
    mov     cx, [si]				; length of its ID string
    cmp     cx, cs:[My_ID_Block]; Q: same length as my ID string?
    jne     next_TSR		;   N: can't be same string then.

    push    di
    push    es			; save pointer to TSR_Info_Struc

    mov     di, cs
    mov     es, di
    mov     di, OFFSET My_Name	; es:di -> my name
    add     si, 2		; ds:si -> it's name
    repe    cmpsb		; Q: same ID string?
    je	    already_installed	;   Y: TSR already in system

    pop     es
    pop     di			; restore es:di -> TSR_Info_Struc

PUBLIC next_TSR
next_TSR:
    les     di, DWORD PTR es:[di.TSR_Next]  ; es:di -> next TSR_Info_Struc
    mov     ax, es
    or	    ax, di		; Q: is there a next TSR?
    jnz     short walk_list	;   Y: go check its ID string
    jmp     Install_TSR 	;   N: go on to next phase

PUBLIC already_installed
already_installed:
    ; Print the program's name. The name is taken from the ID string, and
    ; printed character-by-character (can't use function 9, because it is
    ; not '$'-terminated (it is 0-terminated)
    ;
    ; note: we still have es:di -> TSR_Info_Struc on the stack. Keep for later.
    push    cs
    pop     ds
    mov     si, OFFSET My_Name
    mov     ah, 02h		; int21h, function 2 = Character Output
msg_loop1:
    lodsb
    or	    al, al
    jz	    short test_parameter
    mov     dl, al
    int     21h
    jmp     short msg_loop1

PUBLIC test_parameter
test_parameter:
;
; test if program was called with /r, /u, /d option (case-insensitive)
; Get the command line from the PSP (offset 80h)
;
    mov     es, WORD PTR [TSR_Info.TSR_PSP_Segment]
    mov     di, 0080h			; command line is at offset 80h in PSP
    xor     cx, cx
    mov     cl, BYTE PTR es:[di]	; get length of command line
    mov     al, '/'			; prepare to scan for '/'
    repne   scasb
    jz	    short _test_1		; OK - continue parsing
    jmp     no_params			; scan failed to find a '/'

_test_1:
    cmp     BYTE PTR es:[di], 'r'
    jne     short _test_2		; no match yet
    jmp     reset_request

_test_2:
    cmp     BYTE PTR es:[di], 'R'
    jne     short _test_3		; no match yet
    jmp     reset_request

_test_3:
    cmp     BYTE PTR es:[di], 'd'
    jne     short _test_4
    jmp     toggle_confirm

_test_4:
    cmp     BYTE PTR es:[di], 'D'
    jne     short _test_5
    jmp     toggle_confirm

_test_5:
    cmp     BYTE PTR es:[di], 'u'
    je	    short uninstall

    cmp     BYTE PTR es:[di], 'U'
    je	    short uninstall
    jmp     no_params

PUBLIC uninstall
uninstall:
;
; to uninstall TSR: remove int21h and int2Fh handlers
    pop     ds
    pop     di				; restore ds:di -> TSR_Info_Struc

;
; Check if Windows is running : can't uninstall if it is.
; Why? Because: the data block would disappear, and the Windows App would
; be fiddling with de-allocated memory!
;
    mov     ax, 160Ah
    int     2Fh
    or	    ax, ax			; Q: ax zero? (ie: ax modified?)
    jz	    short cant_uninstall	;   Y: Then Win386 responded
;
; Make sure no other TSR has installed handlers on top of our int21h and int2Fh
; handlers. Otherwise, we can't un-install (can't un-chain our handlers from
; within a list).
;
    mov     ax, 352Fh			; Get address of int2Fh handler
    int     21h
    cmp     bx, OFFSET Int_2Fh_Handler	; Q: offset of int.handler different?
    jne     short other_handlers	;   Y: other TSR installed over ours
    mov     bx, es
    mov     ax, ds
    cmp     bx, ax			; Q: segment of int.handler different?
    jne     short other_handlers	;   Y: other TSR installed over ours

    mov     ax, 3521h			; Get address of int21h handler
    int     21h
    cmp     bx, OFFSET Int_21h_Handler	; Q: offset of int.handler different?
    jne     short other_handlers	;   Y: other TSR installed over ours
    mov     bx, es
    mov     ax, ds
    cmp     bx, ax			; Q: segment of int.handler different?
    jne     short other_handlers	;   Y: other TSR installed over ours

    mov     es, ax			; es = other TSR's code/data segment
    lds     dx, es:[Int_2Fh_Chain_Addr] ; ds:dx -> next int 2F handler
    mov     ax, 252Fh			; set new int2Fh handler
    int     21h

    lds     dx, es:[Int_21h_Chain_Addr] ; ds:dx -> next int 21h handler
    mov     ax, 2521h			; set new int21h handler
    int     21h

    mov     es, es:[di.TSR_PSP_Segment] ; es = PSP (must free)
    mov     ax, 4900h			; free memory block function
    int     21h

    push    cs
    pop     ds
    mov     dx, OFFSET Uninstalled_msg
    jmp     msg_and_terminate

other_handlers:
;
; Other handlers present on top of our handlers. Can't uninstall TSR.
;
    push    cs
    pop     ds
    mov     dx, OFFSET Other_handlers_msg
    jmp     msg_and_terminate

cant_uninstall:
    push    cs
    pop     ds
    mov     dx, OFFSET Cant_uninst_msg
    jmp     msg_and_terminate

PUBLIC toggle_confirm
toggle_confirm:
; /d option: toggle deletion confirmation on and off.
; If callback address is currently 0 (ie: confirm disabled), enable it by
; setting pointer to the call-back.
; Otherwise, zero-out the pointer.

    pop     es
    pop     di				; restore es:di -> TSR_Info_Struc
    ; get pointer to call-back address (located at the end of the data block)
    les     di, DWORD PTR es:[di.TSR_DATA_Block]
    add     di, SIZE_My_Data
    cmp     WORD PTR es:[di], 0 	; Q: is DWORD pointer 0 ?
    jne     short disable_confirm	;   N: checking enabled => disable it
    cmp     WORD PTR es:[di+2], 0	; (2nd part of DWORD pointer)
    jz	    short enable_confirm	;   Y: must enable checking
					;   N: checking enabled => disable it
disable_confirm:
    xor     ax, ax
    mov     WORD PTR es:[di], ax
    mov     WORD PTR es:[di+2], ax
    mov     dx, OFFSET Disable_Check_msg
    jmp     short msg_and_terminate

enable_confirm:
    mov     ax, es
    mov     WORD PTR es:[di+2], ax	; code seg. of routine = data seg. of TSR_Info_Struc
    mov     WORD PTR es:[di], OFFSET Confirm_Delete
    mov     dx, OFFSET Enable_Check_msg
    jmp     short msg_and_terminate


PUBLIC reset_request
reset_request:
    pop     es
    pop     di				; restore es:di -> TSR_Info_Struc
    les     di, DWORD PTR es:[di.TSR_DATA_Block]    ; load ptr to data block
    xor     ax, ax			; reset data block to 0
    mov     cx, 2*256			; 256 DWORDS = 2*256 WORDS
    rep     stosw
    ; print message
    mov     dx, OFFSET Reset_msg

msg_and_terminate:
    mov     ah, 09h		; Display String function
    int     21h
    jmp     terminate		; exit now

PUBLIC no_params
no_params:
;
; no recognized parameter: state that TSR already loaded, and display stats
;
    mov     dx, OFFSET Already_Loaded_msg
    mov     ah, 09h		; Display String function
    int     21h
;
; now print stats on dos calls
;
    pop     es
    pop     di			; es:di -> TSR_Info_Struc
    les     di, DWORD PTR es:[di.TSR_DATA_Block]    ; load ptr to data block
    xor     bx, bx

PUBLIC stats_loop
stats_loop:
    mov     cx, WORD PTR es:[di+bx+2]
    mov     dx, WORD PTR es:[di+bx]
    push    es			    ; save pointer to TSR's Data Block
    push    di
    push    dx			    ; save low word of count
    or	    dx, cx
    jnz     short stats_not_zero
    add     sp, 6		    ; pop dx, di, es
    add     bx, 4
    cmp     bx, 0400h		    ; Q: 256th entry yet? (ie: 256 * 4)
    jne     short stats_loop	    ;	N: continue
    jmp     terminate
stats_not_zero:
    push    cx			    ; save high word of count
    mov     dx, bx		    ; print DOS call #
    shr     dx, 1
    shr     dx, 1
    mov     cx, dx		    ; cx = interrupt number
    mov     ax, cs
    mov     es, ax
    mov     di, OFFSET stats_msg_ah
    call    hex_to_ascii
    pop     dx			    ; restore high word of count
    mov     di, OFFSET stats_msg_count
    call    hex_to_ascii
    pop     dx			    ; restore low word of count
    add     di, 4
    call    hex_to_ascii

    mov     di, OFFSET stats_msg_name
    mov     si, OFFSET Unknown_DOScall		; default string
    cmp     cx, LAST_KNOWN_CALL 		; Q: do we have a name?
    ja	    short stats_copy_name		;   N: go copy default
    mov     si, OFFSET DOScall_Name_Off_Table
    shl     cx, 1				; index into WORD table
    add     si, cx
    mov     si, WORD PTR [si]

PUBLIC stats_copy_name
stats_copy_name:
    lodsb
    or	    al, al
    jz	    short done_name_copy
    stosb
    jmp     short stats_copy_name
done_name_copy:
    mov     al, 13		    ; carriage-return
    stosb
    mov     al, 10		    ; line feed
    stosb
    mov     al, '$'		    ; DOS string terminator
    stosb
    mov     dx, OFFSET stats_msg    ; prepare to print string
    mov     ah, 09h		    ; display string function
    int     21h 		    ; DOS call
    pop     di			    ; restore pointer to TSR's Data Block
    pop     es
    add     bx, 4		    ; next entry
    cmp     bx, 0400h		    ; Q: 256th entry yet? (ie: 256 * 4)
    jne     short stats_loop	    ;	N: continue

PUBLIC terminate
terminate:
;
; DOS Terminate (and NOT stay resident)
;
    mov     ax, 4C00h		; ret code = 0
    int     21h


;
; Install TSR into system
;
PUBLIC Install_TSR
Install_TSR:
;
; get pointer to fully-qualified pathname (in environment)
;
    push    cs
    pop     ds

    mov     es, WORD PTR [TSR_Info.TSR_PSP_Segment]
    mov     ax, WORD PTR es:[02Ch]	    ; environment segment
    mov     es, ax
    xor     di, di
    xor     ax, ax		; scan for a 0
    mov     cx, 0FFFFh		; we don't really want to count
    cld

PUBLIC Scan_env_loop
Scan_env_loop:
    repne   scasb		; do the scan
    cmp     BYTE PTR es:[di], 0 ; Q: another zero (ie: start of argv[0])?
    jne     short Scan_env_loop ;   N: repeat scan

    add     di, 3		; es:di -> full pathname
    push    es
    push    di			; save pointer
;
; Prepare to copy pathname of executable, from environment to our data seg.
;
    mov     dx, cs
    mov     es, dx		; es = our data segment
    mov     di, OFFSET [Exec_Path_Name] ; es:di -> my copy of pathname

    pop     si
    pop     ds			; ds:si -> full pathname

Copy_Pathname:
    lodsb
    or	    ax, ax		    ; Q: zero-terminator?
    jz	    short Done_Copy_Pathname;	Y: done copying
    stosb			    ;	N: write char to our buffer
    jmp     short Copy_Pathname

PUBLIC Done_Copy_Pathname
Done_Copy_Pathname:
;
; free environment block
;
    push    ds
    pop     es			    ; es = environment segment (must free)
    mov     ax, 4900h		    ; free memory block function
    int     21h

    mov     es, dx		    ; es = our data segment
    mov     ds, dx

;
; add a parameter to the pathname: ' /xxxx:xxxx', where xxxx:xxxx is
;   seg:offset of TSR's name
;
    mov     al, ' '		    ; append ' /' to the pathname...
    stosb
    mov     al, '/'
    stosb
    mov     BYTE PTR [di+4], ':'

    mov     si, di
    add     si, 5		    ; si = offset of second parameter
;
; fill in parameters to invoke application - segment:offset of TSR data block
; convert hex to ascii, using table lookup
;
    ; note: dx = data segment, di = offset of 1st param
    call    hex_to_ascii

    mov     dx, OFFSET My_Data_Block
    mov     di, si		    ; offset of second parameter
    call    hex_to_ascii
;
; init pointer to app name in TSR_Info
;
    mov     WORD PTR [TSR_Info.TSR_Exec_Cmd], OFFSET Exec_Path_Name
    mov     WORD PTR [TSR_Info.TSR_Exec_Cmd+2], cs
    mov     WORD PTR [TSR_Info.TSR_Exec_Flags], TSR_WINEXEC
    mov     WORD PTR [TSR_Info.TSR_Exec_Cmd_Show], 4 ;; SW_SHOWNOACTIVATE
    mov     WORD PTR [TSR_Info.TSR_ID_Block], OFFSET My_ID_Block
    mov     WORD PTR [TSR_Info.TSR_ID_Block+2], cs
    mov     WORD PTR [TSR_Info.TSR_Data_Block], OFFSET My_Data_Block
    mov     WORD PTR [TSR_Info.TSR_Data_Block+2], cs
;
; init Win386_startup_info
;
    mov     WORD PTR [Startup_Info.SIS_Instance_Data_Ptr], OFFSET Instance_Info
    mov     WORD PTR [Startup_Info.SIS_Instance_Data_Ptr+2], cs
    mov     WORD PTR [Instance_Info.IIS_Ptr], OFFSET My_Data_Block
    mov     WORD PTR [Instance_Info.IIS_Ptr+2], cs
    mov     WORD PTR [Instance_Info.IIS_Size], SIZE_Data_Block
;
; hook int2f chain
;
    mov     ax, 352Fh
    int     21h
    mov     [Int_2Fh_Chain_Off], bx
    mov     [Int_2Fh_Chain_Seg], es

    mov     dx, OFFSET Int_2Fh_Handler
    mov     ax, 252Fh
    int     21h
;
; hook int21 chain
;
    mov     ax, 3521h
    int     21h
    mov     [Int_21h_Chain_Off], bx
    mov     [Int_21h_Chain_Seg], es

    mov     dx, OFFSET Int_21h_Handler
    mov     ax, 2521h
    int     21h
;
; Indicate that TSR is installed
;
    mov     si, OFFSET My_Name
    mov     ah, 02h		; int21h function 2 - Character Output
msg_loop2:
    lodsb
    or	    al, al
    jz	    short rest_of_msg2
    mov     dl, al
    int     21h
    jmp     short msg_loop2
rest_of_msg2:
    mov     dx, OFFSET First_Load_msg
    mov     ah, 09h		; Display String function
    int     21h
;
; Terminate and Stay Resident
;
    mov     dx, OFFSET Init_Code_Starts
    add     dx, 15			; round up to nearest paragraph
    mov     cl, 4
    shr     dx, cl
    add     dx, WORD PTR [PSP_Size]	; add size (in paras) of PSP
    mov     ax, 3100h
    int     21h

Initialization ENDP

;
; Init-time data
;

PUBLIC PSP_Size
PSP_Size    dw	?

PUBLIC First_Load_msg
First_Load_msg	db  ' is now loaded. Start Windows to activate.'
		db  13,10,'$'

PUBLIC Already_Loaded_msg
Already_Loaded_msg db ' is already installed. Current stats:'
		   db 13,10
		   db 'AH',9,'Hex Count',9,'DOS Function'
		   db 13,10,'$'

PUBLIC Uninstalled_msg
Uninstalled_msg    db ' is uninstalled.', 13,10,'$'

PUBLIC Cant_uninst_msg
Cant_uninst_msg db ' can not be uninstalled while Windows is running.'
		db 13,10,'$'

PUBLIC Other_handlers_msg
Other_handlers_msg   db ' can not be uninstalled: other TSRs present.'
		db 13,10,'$'

PUBLIC Reset_msg
Reset_msg	db ': reset all statistics.'
		db 13,10,'$'

PUBLIC Enable_Check_msg
Enable_Check_msg db ': Deletion checking enabled.'
		db 13,10,'$'

PUBLIC Disable_Check_msg
Disable_Check_msg db ': Deletion checking disabled.'
		db 13, 10, '$'

stats_msg_ah	   db 'xx'
PUBLIC stats_msg
stats_msg	   db 'xx',9
stats_msg_count    db 'xxxxxxxx',9
stats_msg_name	   db 127 dup (?)

include doscalls.asm		    ; table of dos call names

_TEXT	ENDS

    END Initialization
